import cv2
import numpy as np
import matplotlib.pyplot as plt



def check_laplaican(img_file, output_filename):
    img = cv2.imread(img_file, cv2.IMREAD_GRAYSCALE)
    h, w = img.shape

    # Laplacian 연산
    lap = cv2.Laplacian(img, cv2.CV_64F)
    lap_abs = np.abs(lap)

    # 시각화
    plt.figure(figsize=(14, 6))

    plt.subplot(1, 2, 1)
    plt.title("Original Image")
    plt.imshow(img, cmap='gray')
    plt.xlabel("Width (pixels)")
    plt.ylabel("Height (pixels)")
    plt.xticks(np.linspace(0, w, num=5, dtype=int))
    plt.yticks(np.linspace(0, h, num=5, dtype=int))

    plt.subplot(1, 2, 2)
    plt.title("Laplacian (High-Frequency Map)")
    plt.imshow(lap_abs, cmap='gray')
    plt.xlabel("Width (pixels)")
    plt.ylabel("Height (pixels)")
    plt.xticks(np.linspace(0, w, num=5, dtype=int))
    plt.yticks(np.linspace(0, h, num=5, dtype=int))

    plt.tight_layout()
    plt.savefig(output_filename)
    
def check_fft(img_file, output_filename):
    img = cv2.imread(img_file, cv2.IMREAD_GRAYSCALE)
    h, w = img.shape
    f = np.fft.fft2(img)
    fshift = np.fft.fftshift(f)
    magnitude_spectrum = 20 * np.log(np.abs(fshift) + 1e-6)

    # 좌표 중심값
    cx, cy = w // 2, h // 2

    # 시각화
    plt.figure(figsize=(14, 6))

    plt.subplot(1, 2, 1)
    plt.title("Original Image")
    plt.imshow(img, cmap='gray')
    plt.xlabel("Width (pixels)")
    plt.ylabel("Height (pixels)")
    plt.xticks(np.linspace(0, w, num=5, dtype=int))
    plt.yticks(np.linspace(0, h, num=5, dtype=int))

    plt.subplot(1, 2, 2)
    plt.title("FFT Magnitude Spectrum (Frequency Domain)")
    plt.imshow(magnitude_spectrum, cmap='gray')
    plt.xlabel("Frequency (Width direction)")
    plt.ylabel("Frequency (Height direction)")
    plt.xticks([0, cx, w-1], labels=["-f_x", "0", "+f_x"])
    plt.yticks([0, cy, h-1], labels=["-f_y", "0", "+f_y"])

    plt.tight_layout()
    plt.savefig(output_filename)

    
if __name__ == '__main__':
    import os.path as osp 
    
    output_dir = '/HDD/etc/outputs'
    img_file = '/HDD/datasets/projects/Tenneco/Metalbearing/outer/repeatability/1/125032816405615_5/1_image.bmp'
    
    filename = osp.split(osp.splitext(img_file)[0])[-1]

    check_laplaican(img_file, osp.join(output_dir, 'laplacian_' + filename + '.jpg'))
    check_fft(img_file, osp.join(output_dir, 'fft_' + filename + '.jpg'))



import argparse
import collections
import datetime
import glob
import json
import os
import os.path as osp
import sys
import uuid

import imgviz
import cv2
import numpy as np
from typing import Optional
import numpy.typing as npt
import PIL
import math
from tqdm import tqdm
import warnings


try:
    import pycocotools.mask
except ImportError:
    print("Please install pycocotools:\n\n    pip install pycocotools\n")
    sys.exit(1)

def shape_to_mask(img_shape: tuple[int, ...], 
                    points: list[list[float]],
                    shape_type: Optional[str] = None,
                    line_width: int = 10,
                    point_size: int = 5,
                    ignore_shape_types=['point', 'line', 'linestrip']) -> npt.NDArray[np.bool_]:
    
    mask = PIL.Image.fromarray(np.zeros(img_shape[:2], dtype=np.uint8))
    draw = PIL.ImageDraw.Draw(mask)
    xy = [tuple(point) for point in points]
    
    if shape_type not in ignore_shape_types:
        if shape_type == "circle":
            assert len(xy) == 2, "Shape of shape_type=circle must have 2 points"
            (cx, cy), (px, py) = xy
            d = math.sqrt((cx - px) ** 2 + (cy - py) ** 2)
            draw.ellipse([cx - d, cy - d, cx + d, cy + d], outline=1, fill=1)
        elif shape_type == "rectangle":
            assert len(xy) == 2, "Shape of shape_type=rectangle must have 2 points"
            x1, y1 = min(xy[0][0], xy[1][0]), min(xy[0][1], xy[1][1])
            x2, y2 = max(xy[0][0], xy[1][0]), max(xy[0][1], xy[1][1])
            xy = [(x1, y1), (x2, y2)]
            draw.rectangle(xy, outline=1, fill=1)
        elif shape_type == "line":
            assert len(xy) == 2, "Shape of shape_type=line must have 2 points"
            draw.line(xy=xy, fill=1, width=line_width)
        elif shape_type == "linestrip":
            draw.line(xy=xy, fill=1, width=line_width)
        elif shape_type == "point":
            assert len(xy) == 1, "Shape of shape_type=point must have 1 points"
            cx, cy = xy[0]
            r = point_size
            draw.ellipse([cx - r, cy - r, cx + r, cy + r], outline=1, fill=1)
        elif shape_type in [None, "polygon"]:
            if len(xy) <= 2:
                warnings.warn(f'There are less than 2 points with polygon shape-type') 
            else:
                assert len(xy) > 2, "Polygon must have points more than 2"
                draw.polygon(xy=xy, outline=1, fill=1)
        else:
            raise ValueError(f"shape_type={shape_type!r} is not supported.")
        
    return np.array(mask, dtype=bool)


def labelme2coco(input_dir, output_dir, noviz, 
                 assert_image_path, modes, only_json=False, 
                 ignore_shape_types=['point', 'line', 'linestrip']):
    # if osp.exists(args.output_dir):
    #     print("Output directory already exists:", args.output_dir)
    #     sys.exit(1)
    for mode in modes:
        
        if not only_json:
            os.makedirs(osp.join(output_dir, mode), exist_ok=True)
        if not noviz:
            os.makedirs(osp.join(output_dir, mode, "Visualization"), exist_ok=True)
        print("Creating dataset:", output_dir)

        now = datetime.datetime.now()

        data = dict(
            info=dict(
                description=None,
                url=None,
                version=None,
                year=now.year,
                contributor=None,
                date_created=now.strftime("%Y-%m-%d %H:%M:%S.%f"),
            ),
            licenses=[
                dict(
                    url=None,
                    id=0,
                    name=None,
                )
            ],
            images=[
                # license, url, file_name, height, width, date_captured, id
            ],
            type="instances",
            annotations=[
                # segmentation, area, iscrowd, image_id, bbox, category_id, id
            ],
            categories=[
                # supercategory, id, name
            ],
        )

        class_name_to_id = {}
        out_ann_file = osp.join(output_dir, f"{mode}.json")
        label_files = glob.glob(osp.join(input_dir, mode, "*.json"))
        for image_id, filename in tqdm(enumerate(label_files), desc=mode):
            with open(filename, 'r') as jf:
                label_file = json.load(jf)


            
            base = osp.splitext(osp.basename(filename))[0]
            if base == '3_5_124071615341242_12_Outer':
                print(">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>")
            if assert_image_path:
                assert base == osp.splitext(label_file['imagePath'])[0]
            image_ext = osp.splitext(label_file['imagePath'])[-1]
            out_img_file = osp.join(output_dir, mode, base + image_ext)

            img_file = osp.join(input_dir, mode, base + image_ext)
            img = cv2.imread(img_file)
            # img = labelme.utils.img_data_to_arr(label_file.imageData)
            imgviz.io.imsave(out_img_file, img)
            data["images"].append(
                dict(
                    license=0,
                    url=None,
                    # file_name=osp.relpath(out_img_file, osp.dirname(out_ann_file)),
                    file_name=osp.split(osp.splitext(out_img_file)[0])[-1] + osp.splitext(out_img_file)[-1],
                    height=img.shape[0],
                    width=img.shape[1],
                    date_captured=None,
                    id=image_id,
                )
            )

            masks = {}  # for area
            segmentations = collections.defaultdict(list)  # for segmentation
            for shape in label_file['shapes']:
                points = shape["points"]
                label = shape["label"]
                group_id = shape.get("group_id")
                shape_type = shape.get("shape_type", "polygon")
                mask = shape_to_mask(img.shape[:2], points, shape_type, ignore_shape_types=ignore_shape_types)

        
                if label not in class_name_to_id:
                    class_id = len(class_name_to_id) + 1
                    class_name_to_id[label] = class_id
                    data["categories"].append(
                        dict(
                            supercategory=None,
                            id=class_id,
                            name=label,
                        )
                    )

                if group_id is None:
                    group_id = uuid.uuid1()

                instance = (label, group_id)

                if instance in masks:
                    masks[instance] = masks[instance] | mask
                else:
                    masks[instance] = mask

                if shape_type not in ignore_shape_types:
                    if shape_type == "rectangle":
                        (x1, y1), (x2, y2) = points
                        x1, x2 = sorted([x1, x2])
                        y1, y2 = sorted([y1, y2])
                        points = [x1, y1, x2, y1, x2, y2, x1, y2]
                    elif shape_type == "circle":
                        (x1, y1), (x2, y2) = points
                        r = np.linalg.norm([x2 - x1, y2 - y1])
                        # r(1-cos(a/2))<x, a=2*pi/N => N>pi/arccos(1-x/r)
                        # x: tolerance of the gap between the arc and the line segment
                        n_points_circle = max(int(np.pi / np.arccos(1 - 1 / r)), 12)
                        i = np.arange(n_points_circle)
                        x = x1 + r * np.sin(2 * np.pi / n_points_circle * i)
                        y = y1 + r * np.cos(2 * np.pi / n_points_circle * i)
                        points = np.stack((x, y), axis=1).flatten().tolist()
                    else:
                        points = np.asarray(points).flatten().tolist()
                else:
                    points = []

                segmentations[instance].append(points)
            segmentations = dict(segmentations)

            for instance, mask in masks.items():
                cls_name, group_id = instance
                if cls_name not in class_name_to_id:
                    continue
                cls_id = class_name_to_id[cls_name]

                mask = np.asfortranarray(mask.astype(np.uint8))
                mask = pycocotools.mask.encode(mask)
                area = float(pycocotools.mask.area(mask))
                bbox = pycocotools.mask.toBbox(mask).flatten().tolist()

                data["annotations"].append(
                    dict(
                        id=len(data["annotations"]),
                        image_id=image_id,
                        category_id=cls_id,
                        segmentation=segmentations[instance],
                        area=area,
                        bbox=bbox,
                        iscrowd=0,
                    )
                )

            if not noviz:
                viz = img
                if masks:
                    labels, captions, masks = zip(
                        *[
                            (class_name_to_id[cnm], cnm, msk)
                            for (cnm, gid), msk in masks.items()
                            if cnm in class_name_to_id
                        ]
                    )
                    viz = imgviz.instances2rgb(
                        image=img,
                        labels=labels,
                        masks=masks,
                        captions=captions,
                        font_size=15,
                        line_width=2,
                    )
                out_viz_file = osp.join(output_dir, mode, "Visualization", base + image_ext)
                imgviz.io.imsave(out_viz_file, viz)

        with open(out_ann_file, "w") as f:
            json.dump(data, f)


if __name__ == "__main__":
    # parser.add_argument("--input_dir", default='/HDD/datasets/projects/Tenneco/Metalbearing/outer/250110/split_dataset')
    # parser.add_argument("--output_dir", default='/HDD/datasets/projects/Tenneco/Metalbearing/outer/250110/split_coco_dataset')
    input_dir = '/HDD/datasets/projects/Tenneco/Metalbearing/outer/250211/split_dataset'
    output_dir = '/HDD/datasets/projects/Tenneco/Metalbearing/outer/250211/split_coco_dataset'
    noviz = False
    assert_image_path = False
    modes = ['train', 'val']
    only_json = False
    ignore_shape_types = ['point', 'line', 'linestrip']
    
    labelme2coco(input_dir, output_dir, noviz, assert_image_path, modes, only_json=only_json, 
                 ignore_shape_types=ignore_shape_types)


# import cv2 
# from visionsuite.utils.metrics.metrics import get_iou

# def merge_polygons(polygons, threshold=0.25):
    
#     while True:
#         polygon = polygons.pop(0)
        
#         candidate_indexes = []
#         for idx, candidate in enumerate(polygons):
#             iou = get_iou(polygon, candidate, 'polygon')
    
#             if iou > threshold:
#                 candidate_indexes.append(idx)

#         if                 
#         for candidate_index in candidate_indexes:
#             polygons.pop(candidate_index)

from shapely.geometry import Polygon

def close_polygon(polygon):
    if polygon[0] != polygon[-1]:
        polygon.append(polygon[0])
    return polygon

def calculate_iou(polygon1, polygon2):
    poly1 = Polygon(close_polygon(polygon1))
    poly2 = Polygon(close_polygon(polygon2))
    
    if not poly1.is_valid or not poly2.is_valid:
        return 0  # 유효하지 않은 폴리곤은 IoU 0으로 처리
    
    intersection_area = poly1.intersection(poly2).area
    union_area = poly1.union(poly2).area
    
    if union_area == 0:
        return 0
    
    return intersection_area / union_area

def merge_polygons(polygons, threshold=0.5):
    merged_polygons = []
    used = [False] * len(polygons)
    
    for i, polygon1 in enumerate(polygons):
        if used[i]:
            continue
        
        current_poly = Polygon(close_polygon(polygon1))
        for j, polygon2 in enumerate(polygons):
            if i != j and not used[j]:
                iou = calculate_iou(polygon1, polygon2)
                if iou > threshold:
                    current_poly = current_poly.union(Polygon(close_polygon(polygon2)))
                    used[j] = True
        
        merged_polygons.append(list(current_poly.exterior.coords))
        used[i] = True
    
    return merged_polygons


import cv2
import numpy as np
import matplotlib.pyplot as plt


def apply_clahe(img_file, output_filename):
    img = cv2.imread(img_file)  # BGR로 읽힘
    gray = cv2.cvtColor(img, cv2.COLOR_BGR2GRAY)

    # CLAHE 객체 생성 (clipLimit와 tileGridSize는 조절 가능)
    clahe = cv2.createCLAHE(clipLimit=4.0, tileGridSize=(8, 8))
    enhanced = clahe.apply(gray)

    plt.figure(figsize=(12, 4))
    plt.subplot(1, 3, 1)
    plt.title("Original")
    plt.imshow(gray, cmap='gray')
    plt.axis('off')

    plt.subplot(1, 3, 2)
    plt.title("CLAHE")
    plt.imshow(enhanced, cmap='gray')
    plt.axis('off')

    plt.subplot(1, 3, 3)
    plt.title("Difference")
    plt.imshow(cv2.absdiff(enhanced, gray), cmap='hot')
    plt.axis('off')

    plt.tight_layout()
    plt.savefig(output_filename)


def apply_gaussian_noise(img_file, output_filename):
    img = cv2.imread(img_file)  # BGR로 읽힘
    gray = cv2.cvtColor(img, cv2.COLOR_BGR2GRAY)

    noise = np.random.normal(0, 5, gray.shape).astype(np.uint8)
    noisy_img = cv2.add(gray, noise)

    plt.figure(figsize=(12, 4))
    plt.subplot(1, 2, 1)
    plt.title("Original")
    plt.imshow(gray, cmap='gray')
    plt.axis('off')
    
    plt.subplot(1, 2, 2)
    plt.title("Gaussian Noise")
    plt.imshow(noisy_img, cmap='gray')
    plt.axis('off')

    plt.tight_layout()
    plt.savefig(output_filename)

def apply_gaussian_blur(img_file, output_filename):
    img = cv2.imread(img_file)  # BGR로 읽힘
    gray = cv2.cvtColor(img, cv2.COLOR_BGR2GRAY)

    blur = cv2.GaussianBlur(gray, (5, 5), 0)
    highpass = cv2.subtract(gray, blur)


    plt.figure(figsize=(12, 4))
    plt.subplot(1, 2, 1)
    plt.title("Original")
    plt.imshow(gray, cmap='gray')
    plt.axis('off')
    
    plt.subplot(1, 2, 2)
    plt.title("Gaussian Blur")
    plt.imshow(highpass, cmap='gray')
    plt.axis('off')

    plt.tight_layout()
    plt.savefig(output_filename)
    
def apply_edge_filter(img_file, output_filename):
    img = cv2.imread(img_file)  # BGR로 읽힘
    gray = cv2.cvtColor(img, cv2.COLOR_BGR2GRAY)

    edges = cv2.Laplacian(gray, cv2.CV_64F)
    edges = cv2.convertScaleAbs(edges)


    plt.figure(figsize=(12, 4))
    plt.subplot(1, 2, 1)
    plt.title("Original")
    plt.imshow(gray, cmap='gray')
    plt.axis('off')
    
    plt.subplot(1, 2, 2)
    plt.title("Laplacian Edge")
    plt.imshow(edges, cmap='gray')
    plt.axis('off')

    plt.tight_layout()
    plt.savefig(output_filename)
if __name__ == '__main__':
    import os.path as osp 
    
    output_dir = '/HDD/etc/outputs'
    img_file = '/HDD/datasets/projects/Tenneco/Metalbearing/outer/repeatability/1/125032816405615_5/1_image.bmp'
    
    filename = osp.split(osp.splitext(img_file)[0])[-1]

    apply_clahe(img_file, osp.join(output_dir, 'clahe_' + filename + '.jpg'))
    apply_gaussian_noise(img_file, osp.join(output_dir, 'gaussian_noise_' + filename + '.jpg'))
    apply_gaussian_blur(img_file, osp.join(output_dir, 'gaussian_blur_' + filename + '.jpg'))
    apply_edge_filter(img_file, osp.join(output_dir, 'edge_filter_' + filename + '.jpg'))
    




import os 
import os.path as osp 
import glob 
import json 
import cv2
from tqdm import tqdm
from shutil import copyfile
import numpy as np
from visionsuite.utils.helpers import get_filename

from visionsuite.utils.dataset.formats.labelme.utils import get_mask_from_labelme


def labelme2mask(input_dir, output_dir, class2label, input_format, modes, 
                 width=None, height=None, vis=False, output_format='bmp', 
                 rois=[[]], one_class=False, crop={'use': False}):

    if not osp.exists(output_dir):
        os.makedirs(output_dir)

    for mode in modes:
        _output_dir = osp.join(output_dir, mode)
        if not osp.exists(_output_dir):
            os.mkdir(_output_dir)
            
        if vis:
            vis_dir = osp.join(_output_dir, 'vis')
            if not osp.exists(vis_dir):
                os.mkdir(vis_dir)
            
        json_files = glob.glob(osp.join(input_dir, mode, '*.json'))
        
        for json_file in tqdm(json_files, desc=mode):
            img_file = osp.splitext(json_file)[0] + f'.{input_format}'
            img = cv2.imread(img_file)
            filename = get_filename(json_file, False)
            # if filename == '723_1852_124062811280335_3_Outer':
            #     print(">>>>>>>>>>>>>>>>>>>>>>>>>>>>")
            with open(json_file, 'r') as jf:
                anns = json.load(jf)
                
            if width is None and height is None:
                width, height = anns['imageWidth'], anns['imageHeight']
                
            if rois == [[]]:
                rois = [[0, 0, width, height]]
            mask = get_mask_from_labelme(json_file, width, height, class2label, 
                                        format='opencv', one_class=one_class)
            import numpy as np
            mask_output_dir = osp.join(_output_dir, 'masks')
            if not osp.exists(mask_output_dir):
                os.mkdir(mask_output_dir)
                        
            image_output_dir = osp.join(_output_dir, 'images')
            if not osp.exists(image_output_dir):
                os.mkdir(image_output_dir)
                
            for roi in rois:
                roi_img = img[roi[1]:roi[3], roi[0]:roi[2]]
                roi_mask = mask[roi[1]:roi[3], roi[0]:roi[2]]
                
                if not crop['use']:
                    cv2.imwrite(osp.join(mask_output_dir, filename + f'.{output_format}'), roi_mask)
                    cv2.imwrite(osp.join(image_output_dir, filename + f'.{output_format}'), roi_img)
                    
                    if vis:
                        import numpy as np
                        import imgviz 
                        
                        vis_img = np.zeros((roi[3] - roi[1], (roi[2] - roi[0])*2, 3))
                        vis_img[:, :roi[2] - roi[0], :] = roi_img
                        if one_class:
                            vis_img[:, roi_img.shape[1]:, :] = cv2.cvtColor(roi_mask.astype(np.uint8), cv2.COLOR_BGR2RGB)
                        else:
                            color_map = imgviz.label_colormap(50)
                            vis_img[:, roi[2] - roi[0]:, :] = color_map[roi_mask.astype(np.uint8)].astype(np.uint8) 
                        
                        cv2.imwrite(osp.join(vis_dir, filename + '.png'), vis_img)
                else:
                    
                    for ann_id, ann in enumerate(anns['shapes']):

                        if ann['shape_type'] in ['point', 'polygon', 'watershed'] and len(ann['points']) <=2 :
                            continue
                        xs, ys = [], []
                        for point in ann['points']:
                            if point[0] >= roi[0] and point[0] <= roi[2]:
                                xs.append(point[0] - roi[0])
                            elif point[0] < roi[0]:
                                xs.append(0)
                            elif point[0] > roi[2]:
                                xs.append(roi[2] - roi[0])
                            else:
                                raise RuntimeError(
                                    f"Not considered x: point is {point} and roi is {roi}"
                                )

                            if point[1] >= roi[1] and point[1] <= roi[3]:
                                ys.append(point[1] - roi[1])
                            elif point[1] < roi[1]:
                                ys.append(0)
                            elif point[1] > roi[3]:
                                ys.append(roi[3] - roi[1])
                            else:
                                raise RuntimeError(
                                    f"Not considered y: point is {point} and roi is {roi}"
                                )
                                
                        x1, y1, x2, y2 = min(xs), min(ys), max(xs), max(ys)
                        obj_width = max(xs) - min(xs)
                        obj_height = max(ys) - min(ys)
                        case = None
                        if isinstance(crop['offset'], float) and crop['offset'] <= 1 and crop['offset'] >= 0:
                            offset_x, offset_y = (x2 - x1) * crop['offset'], (y2 - y1) * crop['offset']
                            case = 1
                            do_resize = False
                        elif isinstance(crop['offset'], (tuple, list)):
                            assert len(crop['offset']) == 2, ValueError(f"Offset for crop must be 2 values as x and y")
                            offset_x, offset_y = crop['offset'][0], crop['offset'][1]
                            case = 1
                            do_resize = True
                        elif isinstance(crop['offset'], int):
                            offset_x, offset_y = crop['offset'], crop['offset']
                            case = 2
                            do_resize = True
                            
                            if offset_x < obj_width:
                                offset_x = obj_width + 20
                                
                            if offset_y < obj_height:
                                offset_y = obj_height + 20
                            
                        else:
                            raise NotImplementedError(f"NOT yet Considered: {crop['offset']}")
                            
                            
                        if case == 1:
                            x1 = x1 - offset_x if x1 - offset_x >= 0 else 0
                            y1 = y1 - offset_y if y1 - offset_y >= 0 else 0
                            x2 = x2 + offset_x if x1 + offset_x <= roi[2] - roi[0] else roi[2] - roi[0]
                            y2 = y2 + offset_y if y1 + offset_y <= roi[3] - roi[1] else roi[3] - roi[1]
                        elif case == 2:
                            cx, cy = (x2 - x1)/2 + x1, (y2 - y1)/2 + y1
                            x1 = cx - offset_x/2 
                            y1 = cy - offset_y/2 
                            x2 = cx + offset_x/2 
                            y2 = cy + offset_y/2 
                            
                            if x1 < 0:
                                x2 -= x1
                                x1 = 0
                            
                            if x2 > ( roi[2] - roi[0]):
                                x1 -= (x2 - ( roi[2] - roi[0]))
                                x2 = ( roi[2] - roi[0]) 
                            
                            if y1 < 0:
                                y2 -= y1
                                y1 = 0
                            
                            if y2 > (roi[3] - roi[1]):
                                y1 -= (y2 - (roi[3] - roi[1]))
                                y2 = (roi[3] - roi[1]) 
                            
                        _roi_img = roi_img[int(y1):int(y2), int(x1):int(x2)]
                        _roi_mask = roi_mask[int(y1):int(y2), int(x1):int(x2)]
                        
                        if do_resize:
                            _roi_img = cv2.resize(_roi_img, (int(crop['offset']), int(crop['offset'])))
                            _roi_mask = cv2.resize(_roi_mask, (int(crop['offset']), int(crop['offset'])))
                            
                        assert _roi_img.shape[:2] == (int(crop['offset']), int(crop['offset']))
                        assert _roi_mask.shape[:2] == (int(crop['offset']), int(crop['offset']))
                            
                        cv2.imwrite(osp.join(image_output_dir, filename + f'_{ann_id}.{output_format}'), _roi_img)
                        cv2.imwrite(osp.join(mask_output_dir, filename + f'_{ann_id}.{output_format}'), _roi_mask)
                
                        if vis:
                            import numpy as np
                            import imgviz 
                            
                            vis_img = np.zeros((_roi_img.shape[0], _roi_img.shape[1]*2, 3)) # w, h, ch
                            vis_img[:, :_roi_img.shape[1], :] = _roi_img
                            if one_class:
                                vis_img[:, _roi_img.shape[1]:, :] = cv2.cvtColor(_roi_mask.astype(np.uint8), cv2.COLOR_BGR2RGB)
                            else:
                                color_map = imgviz.label_colormap(255)
                                vis_img[:, _roi_img.shape[1]:, :] = color_map[_roi_mask.astype(np.uint8)].astype(np.uint8) 
                            
                            cv2.imwrite(osp.join(vis_dir, filename + f'_{ann_id}.png'), vis_img)
        
            
if __name__ == '__main__':
    input_dir = '/HDD/datasets/projects/Tenneco/Metalbearing/outer/250110/split_dataset'
    output_dir = '/HDD/datasets/projects/Tenneco/Metalbearing/outer/250110/split_mask_dataset_one_class'
    class2label = {'chamfer_mark': 1, 'line': 2, 'mark': 3}
    modes = ['train', 'val']
    # width, height = 1120, 768
    width, height = None, None
    vis = False
    output_format = 'png'
    input_format = 'bmp'
    roi = [[220, 60, 1340, 828]]
    one_class = 255
    crop = {
        'use': True, 'offset': 320 # (x, y),
    }
    
    if crop['use']:
        output_dir += '_crop'


    labelme2mask(input_dir, output_dir, class2label, input_format, modes, width, height, vis, output_format, roi, one_class, crop)

